@@ -1,8 +1,8 @@
-// Copyright (C) 2004, 2006 International Business Machines and others.
+// Copyright (C) 2004, 2008 International Business Machines and others.
 // All Rights Reserved.
 // This code is published under the Common Public License.
 //
-// $Id: IpTripletHelper.cpp 759 2006-07-07 03:07:08Z andreasw $
+// $Id: IpTripletHelper.cpp 1327 2008-09-18 19:01:17Z andreasw $
 //
 // Authors:  Carl Laird, Andreas Waechter     IBM    2004-08-13
 
@@ -20,6 +20,7 @@
 #include "IpZeroMatrix.hpp"
 #include "IpCompoundMatrix.hpp"
 #include "IpCompoundSymMatrix.hpp"
+#include "IpTransposeMatrix.hpp"
 
 #include "IpDenseVector.hpp"
 #include "IpCompoundVector.hpp"
@@ -92,6 +93,11 @@ namespace Ipopt
       return GetNumberEntries_(*cmpd_sym);
     }
 
+    const TransposeMatrix* trans = dynamic_cast<const TransposeMatrix*>(mptr);
+    if (trans) {
+      return GetNumberEntries_(*trans);
+    }
+
     THROW_EXCEPTION(UNKNOWN_MATRIX_TYPE,"Unknown matrix type passed to TripletHelper::GetNumberEntries");
   }
 
@@ -170,6 +176,12 @@ namespace Ipopt
       return;
     }
 
+    const TransposeMatrix* trans = dynamic_cast<const TransposeMatrix*>(mptr);
+    if (trans) {
+      FillRowCol_(n_entries, *trans, row_offset, col_offset, iRow, jCol);
+      return;
+    }
+
     THROW_EXCEPTION(UNKNOWN_MATRIX_TYPE,"Unknown matrix type passed to TripletHelper::FillRowCol");
   }
 
@@ -248,6 +260,12 @@ namespace Ipopt
       return;
     }
 
+    const TransposeMatrix* trans = dynamic_cast<const TransposeMatrix*>(mptr);
+    if (trans) {
+      FillValues_(n_entries, *trans, values);
+      return;
+    }
+
     THROW_EXCEPTION(UNKNOWN_MATRIX_TYPE,"Unknown matrix type passed to TripletHelper::FillValues");
   }
 
@@ -308,6 +326,12 @@ namespace Ipopt
     return n_entries;
   }
 
+
+  Index TripletHelper::GetNumberEntries_(const TransposeMatrix& matrix)
+  {
+    return GetNumberEntries(*matrix.OrigMatrix());
+  }
+
   void TripletHelper::FillRowCol_(Index n_entries, const GenTMatrix& matrix, Index row_offset, Index col_offset, Index* iRow, Index* jCol)
   {
     DBG_ASSERT(n_entries == matrix.Nonzeros());
@@ -359,10 +383,6 @@ namespace Ipopt
     DBG_ASSERT(n_entries == matrix.Dim());
     SmartPtr<const Vector> v = matrix.GetDiag();
     FillValuesFromVector(n_entries, *v, values);
-    //    const DenseVector* vec = dynamic_cast<const DenseVector*>(GetRawPtr(v));
-    //    ASSERT_EXCEPTION(vec, UNKNOWN_VECTOR_TYPE, "Unkown Vector type found in FillValues for DiagMatrix");
-    //    const Number* vals = vec->Values();
-    //    IpBlasDcopy(n_entries, vals, 1, values, 1);
   }
 
   void TripletHelper::FillRowCol_(Index n_entries, const IdentityMatrix& matrix, Index row_offset, Index col_offset, Index* iRow, Index* jCol)
@@ -493,8 +513,8 @@ namespace Ipopt
   {
     Index total_n_entries = 0;
 
-    const CompoundMatrixSpace* owner_space = dynamic_cast<const CompoundMatrixSpace*>(GetRawPtr(matrix.OwnerSpace()));
-    DBG_ASSERT(owner_space);
+    const CompoundMatrixSpace* owner_space = static_cast<const CompoundMatrixSpace*>(GetRawPtr(matrix.OwnerSpace()));
+    DBG_ASSERT(dynamic_cast<const CompoundMatrixSpace*>(GetRawPtr(matrix.OwnerSpace())));
 
     Index c_row_offset = row_offset;
     for (Index i=0; i<matrix.NComps_Rows(); i++) {
@@ -543,8 +563,8 @@ namespace Ipopt
   {
     Index total_n_entries = 0;
 
-    const CompoundSymMatrixSpace* owner_space = dynamic_cast<const CompoundSymMatrixSpace*>(GetRawPtr(matrix.OwnerSpace()));
-    DBG_ASSERT(owner_space);
+    const CompoundSymMatrixSpace* owner_space = static_cast<const CompoundSymMatrixSpace*>(GetRawPtr(matrix.OwnerSpace()));
+    DBG_ASSERT(dynamic_cast<const CompoundSymMatrixSpace*>(GetRawPtr(matrix.OwnerSpace())));
 
     Index c_row_offset = row_offset;
     for (Index i=0; i<matrix.NComps_Dim(); i++) {
@@ -702,6 +722,17 @@ namespace Ipopt
     delete [] jCol;
   }
 
+  void TripletHelper::FillRowCol_(Index n_entries, const TransposeMatrix& matrix, Index row_offset, Index col_offset, Index* iRow, Index* jCol)
+  {
+    FillRowCol(n_entries, *matrix.OrigMatrix(), jCol, iRow,
+               col_offset, row_offset);
+  }
+
+  void TripletHelper::FillValues_(Index n_entries, const TransposeMatrix& matrix, Number* values)
+  {
+    FillValues(n_entries, *matrix.OrigMatrix(), values);
+  }
+
   void TripletHelper::PutValuesInVector(Index dim, const double* values, Vector& vector)
   {
     DBG_ASSERT(dim == vector.Dim());
